import * as Mqtt from 'mqtt'
import * as log from './log'

const MQTT_BROKER_DEFAULT_URL : string = "mqtt://localhost"

type TopicHandlerCallback = (message: Buffer) => void

type TopicHandler = {
    topic: string,
    callback: TopicHandlerCallback|null
}
export class MqttClient {
    private _mqttClient: Mqtt.Client
    private _mqttBrokerUrl: string
    private _topicHandlers: TopicHandler[]

    constructor(mqttBrokerUrl? : string) {
        this._mqttBrokerUrl = (mqttBrokerUrl) ? mqttBrokerUrl : MQTT_BROKER_DEFAULT_URL
        this._topicHandlers = []
    }

    register(topic: string, callback: TopicHandlerCallback|null) : void {
        this._topicHandlers.push({topic, callback})
        log.info(`handler registered for topic ${topic}`)
    }

    exec() : void {
        this._mqttClient = Mqtt.connect(this._mqttBrokerUrl)
        this._mqttClient.on('error', log.error)
        this._mqttClient.on('connect', (): void => {
            log.info("connected to mqtt broker")
            for (let topicHandler of this._topicHandlers) {
                this._mqttClient.subscribe(topicHandler.topic)
            }
        })
        this._mqttClient.on('message', (topic: string, payload: Buffer): void => {
            log.info(`message received, topic ${topic}, payload ${payload}`)
            for (let topicHandler of this._topicHandlers) {
                if (this.topicMatch(topicHandler.topic, topic)) {
                    log.info(`received topic ${topic} matches registered topic ${topicHandler.topic}`)
                }
            }
        })
    }

    private topicMatch(registeredTopic: string, receivedTopic: string) : boolean {
        let registeredTopicFields = registeredTopic.split('/')
        let receivedTopicFields = receivedTopic.split('/')
        for (let field in registeredTopicFields) {
            let regField = registeredTopicFields[field]
            let recvField = receivedTopicFields[field]
            log.info(`recv: ${recvField}, reg: ${regField}`)
            if (regField === "#") {
                log.info('true')
                return true
            }
            if (regField != recvField && regField != "+") {
                log.info('false')
                return false
            }
        }
        if (registeredTopicFields.length == receivedTopicFields.length) {
            log.info('true')
            return true
        } else {
            log.info('false')
            return false
        }

    }
}
